
// DEMO13_4.CPP - 2D trajectory demo
// to compile make sure to include DDRAW.LIB, DSOUND.LIB,
// DINPUT.LIB, WINMM.LIB, and of course the T3DLIB files

// INCLUDES ///////////////////////////////////////////////

#define INITGUID

#define WIN32_LEAN_AND_MEAN  

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>

#include <ddraw.h>  // directX includes
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>
#include <dinput.h>
#include "T3DLIB1.h" // game library includes
#include "T3DLIB2.h"
#include "T3DLIB3.h"

// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINXCLASS"  // class name

#define WINDOW_WIDTH    320  // size of window
#define WINDOW_HEIGHT   240

// define for ducks
#define NUM_DUCKS		 5
//#define NUM_DUCK_FLY_FRAMES  3
//#define NUM_DUCK_DIE_FRAMES  2
#define NUM_DUCK_FRAMES  9
#define MIN_FLIGHT_VEL	 5
#define MAX_FLIGHT_VEL   20
#define MIN_Y_FLIGHT_VEL 1
#define MAX_Y_FLIGHT_VEL 5
#define DUCK_ANIM_SPEED  3

// define duck bob original size
#define DUCK_WIDTH	190
#define DUCK_HEIGHT 125

// define flight regions for ducks (y - coordinates)
#define SMALL_DUCK_REGION_MIN 15
#define SMALL_DUCK_REGION_MAX 45

#define LARGE_DUCK_REGION_MIN 35
#define LARGE_DUCK_REGION_MAX 140

// defines for polygon gun
#define GUN_X0  320			// position of tip of gun
#define GUN_Y0  410
#define BULLET_HWIDTH 2		// bullet half width
#define BULLET_HHEIGHT 2	// bullet half height
#define NUM_PROJECTILES 1	// number of projectiles

// duck types
#define REGULAR 1
#define GOLDEN 2
// MACROS ///////////////////////////////////////////////

#define RAND_RANGE(x,y) ( (x) + (rand()%((y)-(x)+1)))

// TYPES ///////////////////////////////////////////////
// Model the bullet as a projectile for future enhancements
typedef struct PROJ_TYP
{
	int state;      // state 0 off, 1 on
	float x,y;      // position
	float xv, yv;   // velocity

} PROJECTILE, *PROJECTILE_PTR;


// PROTOTYPES /////////////////////////////////////////////

// game console
int Game_Init(void *parms=NULL);
int Game_Shutdown(void *parms=NULL);
int Game_Main(void *parms=NULL);
void Do_Intro();

// missile interface
void Init_Projectiles(void);
void Move_Projectiles(void);
void Draw_Projectiles(void);
void Fire_Projectile(int angle, float vel);

void gun_Sound(void);
void duck_Sound(int index);
void Reset_Duck (int index);
void Kill_Duck (int index);
void Update_Score(int);
// GLOBALS ////////////////////////////////////////////////

HWND main_window_handle   = NULL;	// save the window handle
HINSTANCE main_instance   = NULL;	// save the instance
char buffer[256];					// used to print text

BITMAP_IMAGE intro_bmp;				// holds the background
BITMAP_IMAGE background_bmp;		// holds the background
BOB			duck[NUM_DUCKS];		// the ducks flock

int gun_ids[8];						// sound ids
int duck_sound_ids[NUM_DUCKS];		// Duck sounds
POLYGON2D gun;						// gun drawn as a polygon

PROJECTILE bullets[NUM_PROJECTILES]; // array of bullets
int bullets_available;	// Current number of bullets available
int score;


// Constants in this game but can change later for more real effects
float gravity_force =  0.1;			// gravity
float wind_force    =  0.0;			// wind resistance assumed 0 for bullets

char str[256];						// For printing strings on screen

// FUNCTIONS //////////////////////////////////////////////

LRESULT CALLBACK WindowProc(HWND hwnd, 
						    UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
// this is the main message handler of the system
PAINTSTRUCT	ps;		   // used in WM_PAINT
HDC			hdc;	   // handle to a device context

// what is the message 
switch(msg)
	{	
	case WM_CREATE: 
        {
		// do initialization stuff here
		return(0);
		} break;

    case WM_PAINT:
         {
         // start painting
         hdc = BeginPaint(hwnd,&ps);

         // end painting
         EndPaint(hwnd,&ps);
         return(0);
        } break;

	case WM_DESTROY: 
		{
		// kill the application			
		PostQuitMessage(0);
		return(0);
		} break;

	default:break;

    } // end switch

// process any messages that we didn't take care of 
return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WinProc

// WINMAIN ////////////////////////////////////////////////

int WINAPI WinMain(	HINSTANCE hinstance,
					HINSTANCE hprevinstance,
					LPSTR lpcmdline,
					int ncmdshow)
{
// this is the winmain function

	WNDCLASS winclass;	// this will hold the class we create
	HWND	 hwnd;		// generic window handle
	MSG		 msg;		// generic message
	HDC      hdc;       // generic dc
	PAINTSTRUCT ps;     // generic paintstruct

	// first fill in the window class stucture
	winclass.style			= CS_DBLCLKS | CS_OWNDC | 
							  CS_HREDRAW | CS_VREDRAW;
	winclass.lpfnWndProc	= WindowProc;
	winclass.cbClsExtra		= 0;
	winclass.cbWndExtra		= 0;
	winclass.hInstance		= hinstance;
	winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
	winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName	= NULL; 
	winclass.lpszClassName	= WINDOW_CLASS_NAME;

	// register the window class
	if (!RegisterClass(&winclass))
		return(0);

	// create the window, note the use of WS_POPUP
	if (!(hwnd = CreateWindow(WINDOW_CLASS_NAME, // class
							  "Collision Demo",	 // title
							  WS_POPUP | WS_VISIBLE,
					 		  0,0,	   // x,y
							  WINDOW_WIDTH,  // width
							  WINDOW_HEIGHT, // height
							  NULL,	   // handle to parent 
							  NULL,	   // handle to menu
							  hinstance,// instance
							  NULL)))	// creation parms
	return(0);

	// save the window handle and instance in a global
	main_window_handle = hwnd;
	main_instance      = hinstance;

	// perform all game console specific initialization
	Game_Init();

	// enter main event loop
	while(1)
		{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
			{ 
			// test if this is a quit
			if (msg.message == WM_QUIT)
			   break;
		
			// translate any accelerator keys
			TranslateMessage(&msg);

			// send the message to the window proc
			DispatchMessage(&msg);
			} // end if
    
		// main game processing goes here
		Game_Main();

		} // end while

	// shutdown game and release all resources
	Game_Shutdown();

	// return to Windows like this
	return(msg.wParam);

} // end WinMain

// T3D GAME PROGRAMMING CONSOLE FUNCTIONS ////////////////

int Game_Init(void *parms)
{
// this function is where you do all the initialization 
// for your game

	int index; // looping varsIable
	char filename[80]; // used to build up filenames

	// do the directx initializations
	// hide the mouse
	ShowCursor(FALSE);
	// initialize directinput
	DInput_Init();
	// acquire the keyboard only
	DInput_Init_Keyboard();
	// initilize DirectSound
	DSound_Init();

	// start up DirectDraw (replace the parms as you desire)
	DDraw_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);

	// Runs the Intro
	Do_Intro();

	int duck_fly[5] = {0,1,2,1,0};		// Duck flying anim seq.
	int duck_die[12] = {3,4,5,6,7,8};	// Duck dying anim seq.

	int gduck_fly[5] = {0,1,2,1,0};		// Duck flying anim seq.
	//int gduck_die[12] = {3,4,5,6,7,8};	// Duck dying anim seq.

	// seed random number generate
	srand(Start_Clock());



	// load background image
	Load_Bitmap_File(&bitmap8bit, "BACKGROUND.BMP");
	Create_Bitmap(&background_bmp,0,0,640,480);
	Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit);

	// load the bitmaps
	Load_Bitmap_File(&bitmap8bit, "DUCKFLY3.BMP");

	// create bob
	//Create_BOB(&duck[0],0,120,DUCK_WIDTH,DUCK_HEIGHT,NUM_DUCK_FRAMES,
	//		BOB_ATTR_MULTI_FRAME|BOB_ATTR_MULTI_ANIM|BOB_ATTR_VISIBLE, DDSCAPS_SYSTEMMEMORY);

	Create_BOB(&duck[0],0,120,DUCK_WIDTH,DUCK_HEIGHT,NUM_DUCK_FRAMES,
			BOB_ATTR_MULTI_ANIM|BOB_ATTR_VISIBLE, DDSCAPS_SYSTEMMEMORY);
	
	// load the bob in 
	//for (index=0; index < 3; index++)
	//	Load_Frame_BOB(&duck[0], &bitmap8bit, index, 0, index, BITMAP_EXTRACT_MODE_CELL);

	// load the bob in - first load the duck flying frames - 3
	for (index=0; index < NUM_DUCK_FRAMES; index++)
		Load_Frame_BOB(&duck[0], &bitmap8bit, index, index%3, index/3, BITMAP_EXTRACT_MODE_CELL);

	// now make the remaining NUM_DUCKS clones
	for (index=1; index < NUM_DUCKS; index++)
	{
		// clone the bob
		Clone_BOB(&duck[0], &duck[index]);

		// now here the tricky part, alter the size of the bob, but make sure
		// to keep the aspect ratio the same, so it doesn't look scrunched
		float width  = DUCK_HEIGHT - 10 * index;
		float height = (float)width*((float)DUCK_HEIGHT/(float)DUCK_WIDTH);

		// store new width and height in varsIable cache positions 0,1
		duck[index].varsI[0] = width;
		duck[index].varsI[1] = height;

	} // end for index
	
	// Initialize all the bobs here
	for (index=0; index < NUM_DUCKS; index++) {
		// load all the bob animations
		Load_Animation_BOB (&duck[index], 0, 5, duck_fly);
		Load_Animation_BOB (&duck[index], 1, 6, duck_die);
		
		// set the position of the bob here
		Set_Pos_BOB(&duck[index], 0, RAND_RANGE(LARGE_DUCK_REGION_MIN,LARGE_DUCK_REGION_MAX));
		// set animation speed
		Set_Anim_Speed_BOB(&duck[index], DUCK_ANIM_SPEED);
		// set motion speed. Keep the velocity high here for the ducks to fly fast
		Set_Vel_BOB(&duck[index],RAND_RANGE(MIN_FLIGHT_VEL,MAX_FLIGHT_VEL),
			-1*RAND_RANGE(MIN_Y_FLIGHT_VEL,MAX_Y_FLIGHT_VEL));

		// make animation 0 active
		Set_Animation_BOB (&duck[index], 0);

		// initialize the number of bullets
		bullets_available=50;
	}
	// unload bitmap image
	Unload_Bitmap_File(&bitmap8bit);


	// load background sounds
	gun_ids[0] = DSound_Load_WAV("GUN.WAV");

	// clone sounds
	for (index=1; index < 8; index++)
		gun_ids[index] = DSound_Replicate_Sound(gun_ids[0]);

	// load the duck sounds
	for (index=0; index < NUM_DUCKS; index++) {
		sprintf(str, "DUCKSOUND%d.WAV", index);
		duck_sound_ids[index] = DSound_Load_WAV(str);
	}

	// define points of gun
	VERTEX2DF gun_vertices[4] =  { 3,0, 2,-40, -2,-40, -3,0, };

	// initialize gun
	gun.state       = 1;   // turn it on
	gun.num_verts   = 4;  
	gun.x0          = GUN_X0; // position it
	gun.y0          = GUN_Y0;
	gun.xv          = 0;
	gun.yv          = 0;
	gun.color       = 132; // Black
	gun.vlist       = new VERTEX2DF [gun.num_verts];
 
	for (index = 0; index < gun.num_verts; index++)
		gun.vlist[index] = gun_vertices[index];

	// build the 360 degree look ups
	Build_Sin_Cos_Tables();

	// initialize the bullets
	Init_Projectiles();

	// set clipping rectangle to screen extents so objects dont
	// mess up at edges
	RECT screen_rect = {0,0,screen_width,screen_height};
	lpddclipper = DDraw_Attach_Clipper(lpddsback,1,&screen_rect);

	// set clipping region
	min_clip_x = 0;
	max_clip_x = screen_width - 1;
	min_clip_y = 0;
	max_clip_y = screen_height - 1;

	// return success
	return(1);

} // end Game_Init

///////////////////////////////////////////////////////////

int Game_Shutdown(void *parms)
{
// this function is where you shutdown your game and
// release all resources that you allocated

// shut everything down

	// kill all the bobs
	for (int index=0; index < NUM_DUCKS; index++)
		Destroy_BOB(&duck[index]);

	// shutdown directdraw last
	DDraw_Shutdown();

	// now directsound
	DSound_Stop_All_Sounds();
	DSound_Shutdown();

	// shut down directinput
	DInput_Shutdown();

	// return success
	return(1);

} // end Game_Shutdown

//////////////////////////////////////////////////////////

void gun_Sound(void)
{        
// this functions hunts for an open handle to play a gun sound

	// start a hit sound
	for (int sound_index=0; sound_index < 8; sound_index++)
	{
		// test if this sound is playing
		if (DSound_Status_Sound(gun_ids[sound_index])==0)
		{
		   DSound_Play(gun_ids[sound_index]);
		   break;
		} // end if

	} // end for 

} // end gun_Sound


//////////////////////////////////////////////////////////

void duck_Sound(int index)
{        
// this functions hunts for an open handle to play a duck sound

	// test if this sound is playing
	if (DSound_Status_Sound(duck_sound_ids[index])==0)
	{
	   DSound_Play(duck_sound_ids[index]);
	} // end if

} // end duck_Sound
///////////////////////////////////////////////////////////

void Init_Projectiles(void)
{
// this function initializes the projectiles
	memset(bullets, 0, sizeof(PROJECTILE)*NUM_PROJECTILES);

} // end Init_Projectiles

/////////////////////////////////////////////////////////////

void Move_Projectiles(void)
{
// this function moves all the projectiles and does the physics model
for (int index=0; index<NUM_PROJECTILES; index++)
    {
    if (bullets[index].state==1)
       {
        // translate
        bullets[index].x+=bullets[index].xv;
        bullets[index].y+=bullets[index].yv;

		// test for collision with ducks
		for (int duckIndex=0; duckIndex<NUM_DUCKS; duckIndex++) {
				if (Collision_Test (bullets[index].x, bullets[index].y,
					2*BULLET_HWIDTH, 2*BULLET_HHEIGHT,
					duck[duckIndex].x + duck[duckIndex].width/4,
					duck[duckIndex].y + duck[duckIndex].height/4,
					duck[duckIndex].width/2, duck[duckIndex].height/2))
				{
					bullets[index].state = 0;  // Kill the bullet
					Kill_Duck (duckIndex);
				}
		}

        // test for off screen
        if (bullets[index].x >= screen_width || 
            bullets[index].y >=screen_height || 
            bullets[index].y < 0) 
           {
           // kill the bullet
           bullets[index].state = 0;
           } // end if

        } // end if on
 
    } // end for index

} // end Move_Projectiles
/////////////////////////////////////////////////////////////
void Reset_Duck (int index)
{		
	// set the position of the bob here
	Set_Pos_BOB(&duck[index], 0, RAND_RANGE(LARGE_DUCK_REGION_MIN,LARGE_DUCK_REGION_MAX));
	duck[index].x = -duck[index].width;
	// set animation speed
	Set_Anim_Speed_BOB(&duck[index], DUCK_ANIM_SPEED);
	// set motion speed. Keep the velocity high here for the ducks to fly fast
	// set_Vel_BOB(&duck[index],RAND_RANGE(MIN_FLIGHT_VEL,MAX_FLIGHT_VEL),0);
	Set_Vel_BOB(&duck[index],RAND_RANGE(MIN_FLIGHT_VEL,MAX_FLIGHT_VEL),
		-1*RAND_RANGE(MIN_Y_FLIGHT_VEL,MAX_Y_FLIGHT_VEL));
	// make animation 0 active
	Set_Animation_BOB (&duck[index], 0);
	Move_BOB(&duck[index]);
	Animate_BOB (&duck[index]);
	Show_BOB(&duck[index]);
}
void Kill_Duck (int index)
{		

	// play the duck sound
	duck_Sound(index);
	// set animation speed
	Set_Anim_Speed_BOB(&duck[index], DUCK_ANIM_SPEED);
	// set motion speed. Keep the velocity high here for the ducks to fall down fast
	Set_Vel_BOB(&duck[index],0,10);
	// make animation 1 active
	Set_Animation_BOB (&duck[index], 1);
	Move_BOB(&duck[index]);
	Animate_BOB (&duck[index]);
	Show_BOB(&duck[index]);
	Update_Score(REGULAR);
}

void Update_Score(int type)
{
	switch( type ) 
	{
		case REGULAR:
			{
				score+=10;
				bullets_available+=1;
			}
		case GOLDEN:
			{
				score+=20;
				bullets_available+=10;
			}
	}
}
/////////////////////////////////////////////////////////////

void Draw_Projectiles(void)
{
// this function draws all the projectiles 
// bullet width is 2 and bullet height is 2
for (int index=0; index < NUM_PROJECTILES; index++)
    {
    // is this one alive?
    if (bullets[index].state==1)
       {
       Draw_Rectangle(bullets[index].x-BULLET_HWIDTH,bullets[index].y-BULLET_HHEIGHT, 
                      bullets[index].x+BULLET_HWIDTH,bullets[index].y+BULLET_HHEIGHT,
                      249, lpddsback);
       } // end if

    } // end for index

} // end Draw_Projectiles

/////////////////////////////////////////////////////////////

void Fire_Projectile(int angle, float vel)
{
// this function starts a projectile with the given angle and velocity 
// at the tip of the gun
for (int index=0; index < NUM_PROJECTILES; index++)
    {
    // find an open projectile
    if (bullets[index].state==0)
       {
       // set this missile in motion at the head of gun with the proper angle
       bullets[index].x = gun.vlist[1].x+gun.x0;
       bullets[index].y = gun.vlist[1].y+gun.y0;

       // compute velocity vector based on angle
       bullets[index].xv = vel*cos_look[angle];
       bullets[index].yv = -vel*sin_look[angle];

       // mark as active
       bullets[index].state = 1;

	   // make the gun available again
	   gun.state = 1;		

       // make sound
       gun_Sound();

       // bail
       break;

       } // end if

    } // end for index

} // end Fire_Projectile


void Do_Intro()
{

// the worlds simples intro

int shade; // used as looping var

	for (shade=0; shade < 256; shade++)
	{
		// draw text in shades
		Draw_Text_GDI("Soomar-Darwiche Productions Presents...",320-4*strlen("David Mehi Productions Presents..."),200,
					   RGB(shade,shade,shade),lpddsprimary);
		Sleep(2);
    
		// is user trying to bail
		if (KEY_DOWN(VK_ESCAPE))
			return;

	} // end for shade

	Sleep(1000);

	// load bitmap into primary buffer
	Load_Bitmap_File(&bitmap8bit, "BACKGROUND1.BMP");
	Create_Bitmap(&intro_bmp,0,0,640,480);
	Load_Image_Bitmap(&intro_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
	Set_Palette(bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit);

	// lock back buffer and copy background into it
	DDraw_Lock_Primary_Surface();
	// draw background
	Draw_Bitmap(&intro_bmp, primary_buffer, primary_lpitch,0);
	DDraw_Unlock_Primary_Surface();

	duck_Sound(2);
	
	Sleep(3000);

	// do a screen transition
	Screen_Transitions(SCREEN_GREENNESS,NULL,0);
	Destroy_Bitmap(&intro_bmp);
}
////////////////////////////////////////////////////////////

int Game_Main(void *parms)
{
	// this is the workhorse of your game it will be called
	// continuously in real-time this is like main() in C
	// all the calls for you game go here!
	
	int index;							// looping var
	int soundIndex;						// looping var for sound
	
	static int   curr_angle = 90;		// current angle of elevation from horizontal
	static float curr_vel   = 40;		// current velocity of projectile


	// start the timing clock
	Start_Clock();

	// clear the drawing surface
	//DDraw_Fill_Surface(lpddsback, 0);

	// lock back buffer and copy background into it
	DDraw_Lock_Back_Surface();

	// draw background
	Draw_Bitmap(&background_bmp, back_buffer, back_lpitch,0);

	// do the graphics for the gun
	Draw_Filled_Polygon2D(&gun, back_buffer, back_lpitch);

	// unlock back surface
	DDraw_Unlock_Back_Surface();

	// read keyboard
	DInput_Read_Keyboard();

	// test for rotate
	if ((curr_angle < 180) && keyboard_state[DIK_LEFT]) // rotate left
	{
		   Rotate_Polygon2D_Mat(&gun, -5);
		   curr_angle+=5;
	} // end if
	else if ((curr_angle > 0) &&keyboard_state[DIK_RIGHT]) // rotate right
	{
		   Rotate_Polygon2D_Mat(&gun, 5);
		   curr_angle-=5;
	} // end if

	// test for fire!
	if (keyboard_state[DIK_LCONTROL] & 0x80) 
	{ 
		if (bullets_available <= 0) {
			Draw_Text_GDI("Game is over. Start again.",140, 410, RGB(255,255,255), lpddsback);
		}
		else if (gun.state==1) {
				bullets_available--;
				Fire_Projectile(curr_angle, curr_vel);
				
				// Make the gun unavailable till the shot is cleared
				gun.state = 0;
		}
		else {
				Draw_Text_GDI("LOADING THE GUN ...",140, 410, RGB(255,255,255), lpddsback);
				gun.state = 1;
		}

	} // end fire

	// help
	if (keyboard_state[DIK_LMENU]) 
	{ 
			Draw_Text_GDI("<CTL> to fire the gun, <LEFT>, <RIGHT> to adjust angle",140, 410, RGB(255,255,255), lpddsback);
	} 

	// move all the projectiles
	Move_Projectiles();

	// draw the projectiles
	Draw_Projectiles();

	// move and animate the duck
	for (index=0; index<NUM_DUCKS; index++)
	{
		// test for off the screen wrap around - Reset positions of ducks here
		if (duck[index].x > SCREEN_WIDTH) {
		   Reset_Duck(index);
		}
		
		if (duck[index].y > SCREEN_HEIGHT - 200 || duck[index].y+duck[index].height < 0) {
		   Reset_Duck(index);
		}
		// animate the duck
		Animate_BOB(&duck[index]);
		// move the duck
		Move_BOB(&duck[index]);

	} // end for index

	// draw the ducks
	for (index=NUM_DUCKS-1; index>=0; index--)
		Draw_Scaled_BOB(&duck[index], duck[index].varsI[0], duck[index].varsI[1],lpddsback);

	// draw info texts
	Draw_Text_GDI("<ESC> to Exit",520, 440,RGB(255,255,255), lpddsback);
	Draw_Text_GDI("<ALT> for Help",520, 460,RGB(255,255,255), lpddsback);

	sprintf (str, "BULLETS AVAILABLE : %d", bullets_available);
	Draw_Text_GDI(str,10, 440, RGB(255,255,255), lpddsback);

	sprintf (str, "SCORE : %d", score);
	Draw_Text_GDI(str, 10, 460, RGB(255,255,255), lpddsback);
	
	// flip the surfaces
	DDraw_Flip();

	// sync to 30 fps = 1/30sec = 33 ms
	Wait_Clock(33);

	// check of user is trying to exit
	if (KEY_DOWN(VK_ESCAPE) || keyboard_state[DIK_ESCAPE])
	{
		PostMessage(main_window_handle, WM_DESTROY,0,0);

		// stop all sounds
		DSound_Stop_All_Sounds();

		// do a screen transition
		Screen_Transitions(SCREEN_DARKNESS,NULL,0);
	} // end if

	// return success
	return(1);

} // end Game_Main

//////////////////////////////////////////////////////////